#!/usr/bin/env python3
##
## This file is part of Nolimips, a MIPS simulator with unlimited registers
## Copyright (C) 2003, 2004, 2015 Benoit Perrot <benoit@lrde.epita.fr>
##
## Nolimips is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Nolimips is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
##

import sys, os
import string, re

import nolimips

#### Nolimips' instruction set texi generator --------------

if __name__ != "__main__" or len(sys.argv) != 2:
  print ("Usage: .. SRCDIR")
  sys.exit(1)
srcdir = sys.argv[1]

parser = nolimips.InstructionSetParser()
instructions = parser.parse(sys.stdin)

sys.stdout = open("inst-set.texi.tmp", "w")

print ("""@comment Generated, do not edit by hand.

@node The Nolimips supported instruction set
@section The Nolimips supported instruction set""")

## Menu
print ("@menu")
kind = ""
for inst in instructions:
  if kind != inst.kind:
    kind = inst.kind
    print ("* " + string.capwords(kind) + " instructions ::")
print ("@end menu")

## Subsections
kind = ""
for inst in instructions:
  if kind != inst.kind:
    kind = inst.kind
    print ("@node " + string.capwords(kind) + " instructions")
    print ("@subsection " + string.capwords(kind) + " instructions")

  desc = inst.desc.replace("\n", " ")
  desc = desc.replace("  ", " ")

  print ("@subsubsection " + inst.opcode.upper())
  print ("@findex " + inst.opcode.upper())
  print (desc)

## Epilogue
sys.stdout.close()
sys.stdout = sys.__stdout__
nolimips.lazy_overwrite(srcdir + os.sep + "inst-set.texi", "inst-set.texi.tmp")
os.remove("inst-set.texi.tmp")
